@@ -3,9 +3,10 @@ from requests import get
 from selenium.common.exceptions import TimeoutException
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.by import By
-from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.common.keys import Keys
+from selenium.webdriver.firefox.options import Options
 from selenium.webdriver.remote.webelement import WebElement
+from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.support.ui import WebDriverWait
 from simpler.format import human_seconds, human_bytes
 from sys import stdout
@@ -15,27 +16,28 @@ from traceback import print_exc
 from typing import Dict, Optional, Tuple, Union
 from urllib.request import urlopen
 
-def download_file(url, path=None, chunk_size=10**5):
+def download_file(url, path=None, chunk_size=10**5, show_progress=True) -> str:
 	''' Downloads a file keeping track of the progress. Returns the output path. '''
 	if path is None: path = url.split('/')[-1]
 	r = get(url, stream=True)
 	total_bytes = int(r.headers.get('content-length'))
 	bytes_downloaded = 0
 	start = time()
-	print('Downloading %s (%s)' % (url, human_bytes(total_bytes)))
+	if show_progress: print('Downloading %s (%s)' % (url, human_bytes(total_bytes)))
 	with open(path, 'wb') as fp:
 		for chunk in r.iter_content(chunk_size=chunk_size):
 			if not chunk: continue
 			fp.write(chunk)
-			bytes_downloaded += len(chunk)
-			percent = bytes_downloaded / total_bytes
-			bar = ('█' * int(percent * 32)).ljust(32)
-			time_delta = time() - start
-			eta = human_seconds((total_bytes - bytes_downloaded) * time_delta / bytes_downloaded)
-			avg_speed = human_bytes(bytes_downloaded / time_delta).rjust(9)
-			stdout.flush()
-			stdout.write('\r  %6.02f%% |%s| %s/s eta %s' % (100 * percent, bar, avg_speed, eta))
-	print()
+			if show_progress:
+				bytes_downloaded += len(chunk)
+				percent = bytes_downloaded / total_bytes
+				bar = ('█' * int(percent * 32)).ljust(32)
+				time_delta = time() - start
+				eta = human_seconds((total_bytes - bytes_downloaded) * time_delta / bytes_downloaded)
+				avg_speed = human_bytes(bytes_downloaded / time_delta).rjust(9)
+				stdout.flush()
+				stdout.write('\r  %6.02f%% |%s| %s/s eta %s' % (100 * percent, bar, avg_speed, eta))
+	if show_progress: print()
 	return path
 
 class DownloaderPool:
@@ -92,8 +94,10 @@ class Driver:
 	_CONSOLE_LEVELS = 'debug', 'info', 'log', 'warn', 'error'
 	_YEAR_SECONDS = 365.4 * 24 * 3600
 
-	def __init__(self, timeout: int = 3, keystroke_delay: int = .005, headless: bool = True, disable_flash: bool = True, disable_images: bool = True):
-		self.driver = Firefox(headless=headless, disable_flash=disable_flash, disable_images=disable_images)
+	def __init__(self, timeout: int = 3, keystroke_delay: int = .005, headless: bool = True, disable_flash: bool = True, disable_images: bool = True, language: str = 'en-US, en'):
+		options = Options()
+		options.set_preference('intl.accept_languages', language)
+		self.driver = Firefox(headless=headless, disable_flash=disable_flash, disable_images=disable_images, options=options)
 		self.timeout = timeout
 		self.keystroke_delay = keystroke_delay
 
@@ -116,11 +120,10 @@ class Driver:
 
 	def select(self, element: Union[str, WebElement], wait: bool = True, all: bool = False, raise_errors: bool = None) -> WebElement:
 		if isinstance(element, WebElement): return element
-		if wait:
-			found = self.wait(element, raise_errors=not all if raise_errors is None else raise_errors)
+		found = self.wait(element, raise_errors=not all if raise_errors is None else raise_errors) if wait else False
 		if all:
 			return self.driver.find_elements_by_css_selector(element)
-		elif not wait or found:
+		elif found:
 			return self.driver.find_element_by_css_selector(element)
 
 	def write(self, element: Union[str, WebElement], text: str, clear: bool = False) -> None:
@@ -154,6 +157,9 @@ class Driver:
 			deltaX: arguments[1], deltaY: arguments[2], clientX: arguments[3], clientY: arguments[4]
 		}))''', self.select(element), x_delta, y_delta, mouse_x, mouse_y)
 
+	def scroll_into_view(self, element: Union[str, WebElement]):
+		self.driver.execute_script('arguments[0].scrollIntoView({"block":"center"});', self.select(element))
+
 	def attribute(self, element: Union[str, WebElement], attribute: str, value: Optional[str] = None) -> Optional[str]:
 		args = [self.select(element), attribute]
 		if value is None:
